from PyUniversalKit import CSVkit,Netkit
from lxml import etree

class Proxy():
    def __init__(self):
        self.rules = {
            "XPATH":'//div[@class="container"]/div[@class="containerbox boxindex"]/div[@class="layui-row layui-col-space15"]/div[@align="center"]/table/tr/td/text()',
        }
    def _analysis_xpath(self,url:str,nums):
        rules = self.rules["XPATH"]
        url_response = Netkit.get(url=url,encoding="gb2312").text
        to_html = etree.HTML(url_response)
        list_ip = to_html.xpath(rules)
        num = len(list_ip[5:])
        last_list = []
        for i in range(0,num,5):
            try:
                last_list.append([list_ip[5:][i],list_ip[5:][i+1],list_ip[5:][i+2],list_ip[5:][i+3],list_ip[5:][i+4]])
            except IndexError:
                pass
        CSVkit.write(PATH="./static/proxies%s.csv"%nums,header=['ip','端口号','代理位置','代理类型','验证时间'],rows=last_list)


"""
    
    You can use the following function to get all the IPs in the url: http://www.66ip.cn/%s.html

    print("loading...")
    for j in range(2,2453):
        try:
            Proxy()._analysis_xpath(url='http://www.66ip.cn/%s.html' % j, nums=j)
        except:
            pass
        
"""





from . import test,USER_AGENT_LIST
from .models import TestResponse
from typing import Dict,Union

def get(url:str,**kwargs) -> TestResponse:
    """

    :param url: Str
    :param kwargs: encoding,cookie
    :return: TestResponse
    """
    if "encoding" not in kwargs:
        kwargs.setdefault("encoding", "utf-8")
    if "cookie" not in kwargs:
        kwargs.setdefault("headers", USER_AGENT_LIST.returnUA(None))
    else:
        kwargs.setdefault("headers", USER_AGENT_LIST.returnUA(kwargs["cookie"]))
        del kwargs["cookie"]

    return test.Test().get(url, **kwargs)


def post(url:str,data:Dict[str,Union[str,int,float,bytes]],**kwargs) -> TestResponse:
    """

    :param url: Str
    :param data: Dict
    :param kwargs: encoding,cookie
    :return: TestResponse
    """
    if "encoding" not in kwargs:
        kwargs.setdefault("encoding", "utf-8")
    if "cookie" not in kwargs:
        kwargs.setdefault("headers", USER_AGENT_LIST.returnUA(None))
    else:
        kwargs.setdefault("headers", USER_AGENT_LIST.returnUA(kwargs["cookie"]))
        del kwargs["cookie"]

    return test.Test().post(url, data, **kwargs)












"""
所有的定义对象
"""

#在Response类基础上
class TestResponse(object):
    def __init__(self,_content):
        self._content = _content

    @property
    def text(self):
        try:
            return self._content.text
        except Exception as e:
            return "error:%s"%str(e)

    @property
    def json(self):
        try:
            return self._content.json()
        except Exception as e:
            return "error:%s\nPossible reason:The data itself is not in json format"%str(e)

    @property
    def content(self):
        try:
            return self._content.content
        except Exception as e:
            return "error:%s"%str(e)

    @property
    def next(self):
        try:
            return self._content.next
        except Exception as e:
            return "error:%s"%str(e)

    @property
    def links(self):
        try:
            return self._content.links
        except Exception as e:
            return "error:%s"%str(e)








from .models import DefaultSetting
from PyUniversalKit import Netkit
import re

class Interface():
    def __init__(self):
        self.rule = [
            # Matching formats such as (1)
            r'[(（]/d+[)）]',
            # Matches such as 1. 1:
            r'\d+[.。:]',
            # You can add more ...

        ]
    # You can add more interface

    @staticmethod
    def iterator(end_num):
        num = 0
        while num < end_num:
            yield num
            num += 1

    @property
    def all_rule(self) -> str:
        return "|".join(self.rule)

    def __remove_label(self,input_str:str,rule:str) -> str:
        get_removed = re.findall(rule,input_str)
        if len(get_removed) == 0:
            return input_str
        else:
            for i in get_removed:
                input_str = input_str.replace(i,"")
            return input_str

    def _icodef(self,Q_list:list,cookie:str) -> list:
        """

        :param Q_list: List of questions extracted from the page
        :param cookie: User input or default cookie
        :return: Type of PrettyTable Object

        * 慕课小工具 Interface ,Project address:`https://cx.icodef.com/query.html?`

        """

        # _return_list is a [[]] list type
        _return_list:list = []

        for each_q in Q_list:
            # -----start define-----
            _Standard_answer:list = []

            # Remove number
            up_q = self.__remove_label(each_q,self.all_rule)

            # Find the range of text
            Scope = len(up_q)

            # Options
            _option: str = ""

            # Content
            _content: str = ""

            # Indent
            indentation = 1/3

            # -----end define-----

            """
            Search for the incoming data, the search result is empty or
            error will be repeated three times, each time reducing the current data
            If there is no result after three attempts, it will be skipped automatically.
            """

            for loop in self.iterator(3):
                data = {
                    DefaultSetting().api_icodef[-1][0]:up_q[0:Scope]
                }
                try:
                    anwser = Netkit.post(url=DefaultSetting().api_icodef[0], cookie=cookie, data=data).json[0]["result"][0]
                    for each_simple_anwser in anwser["correct"]:

                        if type(each_simple_anwser["option"]) == bool:
                            flag = str(each_simple_anwser["option"])
                            _option += flag
                            _content += anwser["topic"]
                        elif "<img" in each_simple_anwser["content"]:
                            _option += each_simple_anwser["option"]
                            reg = r'src="(.*?)"'
                            result = "https://cx.icodef.com%s"%"".join(re.findall(reg,each_simple_anwser["content"]))
                            _content += "[%s]"%result
                        else:
                            _option += each_simple_anwser["option"]
                            if each_simple_anwser["content"] == "":
                                _content = "None"
                            else:
                                _content += "%s,"%each_simple_anwser["content"]

                    _content = re.sub('<[^<+]+?>', '', _content)
                    for (key, val) in DefaultSetting().replace_rules.items():
                        _content = _content.replace(key, val)
                    break

                except:
                    if loop < 3:
                        Scope = int(indentation * Scope)
                        continue
                    else:
                        anwser = "Format not supported or search error"
                        _option = anwser
                        _content = anwser

            _return_list.append(["%s..."%up_q[0:8],_option, _content])


        return _return_list


import requests,os
from .models import TestResponse
from typing import Union,Dict

class Test():
    def get(self,url:str,headers:Dict[str,Union[str,int,float,bytes]],encoding:str) -> Union[TestResponse,str]:
        try:
            response = requests.get(url=url, headers=headers)
            response.encoding = encoding
            return TestResponse(_content=response)
        except Exception as e:
            return "error:%s" % str(e)

    def post(self,url:str,data:Dict[str,Union[str,int,float,bytes]],headers:Dict[str,Union[str,int,float,bytes]],encoding:str) -> Union[TestResponse,str]:
        try:

            proxies = {
                "http": "http://82.114.93.210:8080"
            }
            response = requests.post(url=url, headers=headers, data=data,proxies=proxies)
            response.encoding = encoding
            return TestResponse(_content=response)
        except Exception as e:
            return "error:%s" % str(e)














from PyUniversalKit import Netkit
from .models import DefaultSetting,Match
from PyUniversalKit.Netkit.models import TestResponse
from .struct import AnswerObject
from .error import MatchError,ApiTypeError,MatchTypeError,ThirdPackageImportError
from .interface import Interface
from typing import Dict
import re
try:
    from lxml import etree
except:
    raise ThirdPackageImportError("lxml")




class QSobject():

    def search_question(self,url:str,**kwargs) -> AnswerObject:
        def __get_Q_list(which: Dict, add_list: list,url_response:TestResponse) -> list:
            add_list.clear()
            kind = which["kind"]
            reg = which["reg"]
            if kind == "RE":
                # RE Type Matching Rules
                url_result = re.findall(reg, url_response.text)
                for each_result in url_result:
                    to_string = "".join(each_result[:])
                    drop_html = re.sub('<[^<+]+?>', '', to_string)

                    # First data filtering rules

                    for (key, val) in DefaultSetting().replace_rules.items():
                        drop_html = drop_html.replace(key, val)
                    add_list.append(drop_html)
                return add_list
            elif kind == "XPATH":
                # XPATH Matching Rules
                url_response = Netkit.get(url=url).text
                to_html = etree.HTML(url_response)
                url_result = to_html.xpath(reg)
                for each_result in url_result:
                    to_string = "".join(each_result[:])
                    drop_html = re.sub('<[^<+]+?>', '', to_string)

                    # First data filtering rules

                    for (key, val) in DefaultSetting().replace_rules.items():
                        drop_html = drop_html.replace(key, val)
                    add_list.append(drop_html)
                return add_list

            # Here you can add the corresponding data extraction method for the new match type
            else:

                raise MatchTypeError(kind, reg)




        API_type = kwargs["API"]
        cookie = kwargs["cookie"]

        # Call the properties of the match rule class to get all supported match rules
        Matchs = Match().return_all_match

        # Get HTML resource objects with the Net Toolkit
        url_response = Netkit.get(url=url,cookie=cookie)

        # Iterate through all the rules, break when a rule is feasible, compose
        # a list of filtered data and pass it to the _icodef function, which will
        # iterate through and search that list

        for every_matches in Matchs:
            # Return List
            return_match_list = __get_Q_list(every_matches,[],url_response)
            if len(return_match_list) == 0 or "" in return_match_list:
                pass
            else:

                if API_type == "icodef":
                    Anwser_list = Interface()._icodef(return_match_list, cookie)
                    return AnswerObject(_content=Anwser_list,_kind="icodef")


                # add more api type here
                else:
                    raise ApiTypeError(API_type)
        else:
            raise MatchError(_url=url)














from typing import Dict
from PyUniversalKit import CSVkit
from .error import AdminRunError
"""
---------------------------------------------------------------------------------------------
The models.py file holds the project matching rules and the default settings
In the match rules class you can add new regular expressions and decide whether they need to be returned or not.
In the default settings class you can customize methods or properties to access the privacy settings data, plus you can set and add.

* First data screening rules

* Default interface

* List of supported interfaces

* Default Cookie
---------------------------------------------------------------------------------------------

"""

class Match(object):
    def __init__(self):
        # Custom rules need to be added to this again
        self.all_return = [self.__general(),self.__discussion_1(),self.__discussion_2()]

    @staticmethod
    def __general() -> Dict:
        kind = "RE"
        reg = r'<h3 class="mark_name colorDeep">([\S\s]*?)</h3>'
        reg_dict = {
            "kind":kind,
            "reg":reg
        }
        return reg_dict


    @staticmethod
    def __discussion_1() -> Dict:
        kind = "RE"
        reg = r'"content":"(.*?)"'
        reg_dict = {
            "kind": kind,
            "reg": reg
        }
        return reg_dict

    @staticmethod
    def __discussion_2() -> Dict:
        kind = "RE"
        reg = r'"title":"(.*?)"'
        reg_dict = {
            "kind": kind,
            "reg": reg
        }
        return reg_dict

    # Add new matching rules here (via decorators)


    @property
    def return_all_match(self) -> list:
        return self.all_return


class DefaultSetting(object):
    def __init__(self):
        self.setting = {
            "default_API": "icodef",
            "API_Support": ["icodef"],
            "default_cookie_Chaoxing": "Add your own cookie",
            # Add more setting here
        }

        # First data filtering rules -- RE
        """
        self.re_replace_rules = [
            # Matching (1) class
            r'[(（\[]/d+[)）\]]'
        ]
        """

        # First data filtering rules
        self.replace_rules = {
            "\n": "",
            "&nbsp;": "",
            " ": "",
            "(单选题)": "",
            "(多选题)": "",
            "(填空题)": "",
            "(判断题)": "",
            "[判断题]": "",
            "(简答题)": "",
            "(计算题)": "",
            "(单选题,": "",
            "(单选题": "",
            "(其它)": "",
            "(10分)": "",
            "10分)": "",
            "10分": "",
            "简答题":"",
            "(简答题":"",
            "*":"",
            "&": "",
            "@": "",
            "(":"",
            ")": "",
            # Add more rules here to

        }

    @property
    def default_api(self) -> str:
        return self.setting["default_API"]

    @property
    def default_cookie_Chaoxing(self) -> str:
        return self.setting["default_cookie_Chaoxing"]

    @property
    def api_support(self) -> list:
        return self.setting["API_Support"]

    @property
    def api_icodef(self) -> list:
        # API address and rules
        # [API address , [params of post...]]
        return ["https://cx.icodef.com/v2/answer",["topic[0]"]]

    # Add more API interface here




class Mapping():

    # This method will be loaded every time it is run
    def setting_mapping(self,api_type,header:list) -> bool:
        try:
            CSVkit.write(PATH="%s.csv" % api_type, header=header, rows=[[]])
            return True
        except:
            raise AdminRunError(api_type)








import setuptools
from setuptools import setup,find_packages

setuptools.setup(
    name = "PyUniversalKit",
    version = "1.0.3",
    author = "MatthewYt",
    author_email = "2158707744@qq.com",
    description = "Hello,PyUniversalKit!",
    packages = find_packages(),
    url='https://github.com/yangtang-special/PyUniversalKit',
    long_description_content_type = "text/markdown",
    data_files = ["/Users/macos/PycharmProjects/Machine Learning/README.md"]

)




from .operate import Operate
from .error import CSVkitInputTypeError,CSVkitImportError

def read(PATH:str,**kwargs) -> Operate():
    """

    :param PATH: Str
    :param kwargs: The incoming parameters are path [Reading_method:str,...]
    :return: Operate() Object
    """

    if 'Reading_method' not in kwargs:
        kwargs.setdefault("Reading_method","r")
    return Operate().read(PATH,**kwargs)

def write(PATH:str,header:list,rows:list,**kwargs) -> Operate():
    """
    :param PATH: Str
    :param header: Single-dimensional array -> []
    :param rows: Multidimensional Arrays -> [[Optional]]
    :param kwargs: [Writing_method:Bool,...]
    :return: Operate() Object

    ========= ======================================================================================================
    CSV       读写模式
    --------- ------------------------------------------------------------------------------------------------------
    'r'       打开只读文件 该文件必须存在
    'r+'      打开可读写的文件，该文件必须存在。
    'w'       打开只写文件，若文件存在则文件长度清为0，即该文件内容会消失。若文件不存在则建立该文件。
    'w+'      打开可读写文件，若文件存在则文件长度清为零，即该文件内容会消失。若文件不存在则建立该文件。
    'a'       以附加的方式打开只写文件。若文件不存在，则会建立该文件，如果文件存在，写入的数据会被加到文件尾，即文件原先的内容会被保留。
    'a+'      以附加方式打开可读写的文件。若文件不存在，则会建立该文件，如果文件存在，写入的数据会被加到文件尾后，即文件原先的内容会被保留。
    ========= ======================================================================================================

    Issues to note about this function

    * The only necessary parameters are the first three, see `operate.py` for
      details of how to operate the function.

    * This function returns an object of the operation type, processes the
      relevant data in the object's built-in functions and returns the
      WriteCSVobject object

    """

    if "Writing_method" not in kwargs:
        kwargs.setdefault("Writing_method","w+")

    if type(rows[0]) != list:
        raise CSVkitInputTypeError(rows)
    else:
        return Operate().write(PATH, header, rows, **kwargs)